@@ -50,6 +50,10 @@ EFI_GUID gIntelDimmPbrTagIdVariableguid = INTEL_DIMM_PBR_TAGID_VARIABLE_GUID;
 EFI_GUID gDcpmmProtocolGuid = EFI_DCPMM_GUID;
 #endif // !OS_BUILD
 
+#ifdef OS_BUILD
+extern UINT8* gSmbiosTable;
+#endif // OS_BUILD
+
 /**
   Array of dimms UEFI-related data structures.
 **/
@@ -210,7 +214,9 @@ CleanNamespacesAndISs(
   )
 {
   EFI_STATUS ReturnCode = EFI_SUCCESS;
-#ifndef OS_BUILD
+  DIMM *pDimm = NULL;
+  LIST_ENTRY *pDimmNode = NULL;
+
   ReturnCode = CleanNamespaces();
   if (EFI_ERROR(ReturnCode)) {
     goto Finish;
@@ -218,6 +224,11 @@ CleanNamespacesAndISs(
 
   CleanNamespacesList(&gNvmDimmData->PMEMDev.Namespaces);
 
+  LIST_FOR_EACH(pDimmNode, &gNvmDimmData->PMEMDev.Dimms) {
+    pDimm = DIMM_FROM_NODE(pDimmNode);
+    FreeLsaSafe(&pDimm->pLsa);
+  }
+
   /**
     Remove Interleave Sets
   **/
@@ -228,7 +239,6 @@ CleanNamespacesAndISs(
   gNvmDimmData->PMEMDev.RegionsNfitInitialized = FALSE;
 
 Finish:
-#endif
   return ReturnCode;
 }
 
@@ -416,10 +426,6 @@ NvmDimmDriverUnload(
   }
 
 
-  /**
-    clean up data struct
-  **/
-  FREE_POOL_SAFE(gNvmDimmData);
 
   if (EFI_ERROR(ReturnCode) && DriverAlreadyUnloaded) {
     NVDIMM_WARN("The driver was not properly initialized or was unloaded before, error = " FORMAT_EFI_STATUS ".", TempReturnCode);
@@ -427,6 +433,13 @@ NvmDimmDriverUnload(
     ReturnCode = EFI_SUCCESS;
   }
 #endif
+
+
+  /**
+    clean up data struct
+  **/
+  FREE_POOL_SAFE(gNvmDimmData);
+
   NVDIMM_DBG("Exiting DriverUnload, error = " FORMAT_EFI_STATUS ".\n", ReturnCode);
   NVDIMM_EXIT_I64(ReturnCode);
   return ReturnCode;
@@ -1433,6 +1446,7 @@ NvmDimmDriverDriverBindingStart(
          ReturnCode = EFI_OUT_OF_RESOURCES;
          goto Finish;
       }
+      Index++;
    }
 
  Finish:
@@ -1658,7 +1672,7 @@ NvmDimmDriverDriverBindingStop(
   )
 {
   EFI_STATUS ReturnCode = EFI_SUCCESS;
-#ifndef OS_BUILD
+
   EFI_STATUS TempReturnCode = EFI_SUCCESS;
   DIMM *pDimm = NULL;
   LIST_ENTRY *pDimmNode = NULL;
@@ -1709,6 +1723,8 @@ NvmDimmDriverDriverBindingStop(
       called more than once.
     **/
     if (gDimmsUefiData[Index].DeviceHandle != NULL) {
+
+#ifndef OS_BUILD
       ReturnCode = gBS->CloseProtocol(
         ControllerHandle,
         &gEfiDevicePathProtocolGuid,
@@ -1732,8 +1748,10 @@ NvmDimmDriverDriverBindingStop(
       if (EFI_ERROR(ReturnCode)) {
         NVDIMM_WARN("Failed to uninstall the child device device path protocol, error = " FORMAT_EFI_STATUS ".\n", ReturnCode);
       }
+#endif
 
       if (IsDimmManageable(pDimm)) {
+#ifndef OS_BUILD
         ReturnCode = gBS->UninstallMultipleProtocolInterfaces(
           gDimmsUefiData[Index].DeviceHandle,
           &gNvmDimmFirmwareManagementProtocolGuid, &gDimmsUefiData[Index].FirmwareManagementInstance,
@@ -1763,22 +1781,26 @@ NvmDimmDriverDriverBindingStop(
         if (EFI_ERROR(ReturnCode)) {
           NVDIMM_WARN("Failed to uninstall the child device NVDIMM label protocol, error = " FORMAT_EFI_STATUS ".\n", ReturnCode);
         }
+#endif
       }
       gDimmsUefiData[Index].DeviceHandle = NULL;
 
-      if (gDimmsUefiData[Index].pDevicePath != NULL) {
-        gBS->FreePool(gDimmsUefiData[Index].pDevicePath);
-        gDimmsUefiData[Index].pDevicePath = NULL;
-      }
+    }
 
-      if (gDimmsUefiData[Index].pDimmName != NULL) {
-        FreeUnicodeStringTable(gDimmsUefiData[Index].pDimmName);
-        gDimmsUefiData[Index].pDimmName = NULL;
-      }
+    if (gDimmsUefiData[Index].pDevicePath != NULL) {
+      FreePool(gDimmsUefiData[Index].pDevicePath);
+      gDimmsUefiData[Index].pDevicePath = NULL;
+    }
+
+    if (gDimmsUefiData[Index].pDimmName != NULL) {
+      FreeUnicodeStringTable(gDimmsUefiData[Index].pDimmName);
+      gDimmsUefiData[Index].pDimmName = NULL;
     }
     Index++;
   }
 
+
+#ifndef OS_BUILD
   ReturnCode = gBS->CloseProtocol(
     ControllerHandle,
     &gEfiDevicePathProtocolGuid,
@@ -1817,6 +1839,8 @@ NvmDimmDriverDriverBindingStop(
   if (EFI_ERROR(TempReturnCode)) {
     NVDIMM_DBG("Failed to Smbus deinit, error = " FORMAT_EFI_STATUS ".\n", TempReturnCode);
   }
+#endif
+
   /**
     Remove the DIMM from memory
   **/
@@ -1834,6 +1858,11 @@ NvmDimmDriverDriverBindingStop(
   /** Free PMTT tables memory **/
   FreeParsedPmtt(&gNvmDimmData->PMEMDev.pPmttHead);
 
+#ifdef OS_BUILD
+  // Free an OS-only reference to the smbios table too
+  FREE_POOL_SAFE(gSmbiosTable);
+#endif
+
   if (gNvmDimmData->HiiHandle != NULL) {
     HiiRemovePackages(gNvmDimmData->HiiHandle);
     gNvmDimmData->HiiHandle = NULL;
@@ -1843,24 +1872,7 @@ NvmDimmDriverDriverBindingStop(
   gNvmDimmData->ControllerHandle = NULL;
 
 Finish:
-#else //not OS_BUILD
 
-
-  /**
-   Remove the DIMM from memory
- **/
-  ReturnCode = FreeDimmList();
-
-  /** Free PCAT tables memory **/
-  FreeParsedPcat(&gNvmDimmData->PMEMDev.pPcatHead);
-
-  /** Free NFIT tables memory **/
-  FreeParsedNfit(&gNvmDimmData->PMEMDev.pFitHead);
-
-  /** Free PMTT tables memory **/
-  FreeParsedPmtt(&gNvmDimmData->PMEMDev.pPmttHead);
-
-#endif //not OS_BUILD
 #if _BullseyeCoverage
 #ifndef OS_BUILD
   cov_dumpData();
